@@ -50,22 +50,24 @@
             [com.puppetlabs.jdbc :as pl-jdbc]
             [com.puppetlabs.jetty :as jetty]
             [com.puppetlabs.mq :as mq]
-            [com.puppetlabs.utils :as pl-utils]
+            [puppetlabs.kitchensink.core :as kitchensink]
             [clojure.java.jdbc :as sql]
             [clojure.string :as string]
             [clojure.tools.logging :as log]
-            [cheshire.core :as json]
-            [com.puppetlabs.puppetdb.http.server :as server])
+            [com.puppetlabs.cheshire :as json]
+            [com.puppetlabs.puppetdb.http.server :as server]
+            [com.puppetlabs.puppetdb.config :as conf]
+            [com.puppetlabs.puppetdb.utils :as utils])
   (:use [clojure.java.io :only [file]]
         [clj-time.core :only [ago secs minutes days]]
-        [clojure.core.incubator :only (-?>)]
         [overtone.at-at :only (mk-pool interspaced)]
         [com.puppetlabs.time :only [to-secs to-millis parse-period format-period period?]]
         [com.puppetlabs.jdbc :only (with-transacted-connection)]
-        [com.puppetlabs.utils :only (cli! configure-logging! inis-to-map with-error-delivery)]
+        [puppetlabs.kitchensink.core :only (cli! with-error-delivery)]
         [com.puppetlabs.repl :only (start-repl)]
         [com.puppetlabs.puppetdb.scf.migrate :only [migrate!]]
-        [com.puppetlabs.puppetdb.version :only [version update-info]]))
+        [com.puppetlabs.puppetdb.version :only [version update-info]]
+        [com.puppetlabs.puppetdb.command.constants :only [command-names]]))
 
 (def cli-description "Main PuppetDB daemon")
 
@@ -85,14 +87,15 @@
   This function doesn't terminate. If we encounter an exception when
   processing commands from the message queue, we retry the operation
   after reopening a fresh connection with the MQ."
-  [mq mq-endpoint discard-dir db]
-  (pl-utils/keep-going
+  [mq mq-endpoint discard-dir opt-map]
+  {:pre [(:db opt-map)]}
+  (kitchensink/keep-going
    (fn [exception]
      (log/error exception "Error during command processing; reestablishing connection after 10s")
      (Thread/sleep 10000))
 
    (with-open [conn (mq/connect! mq)]
-     (command/process-commands! conn mq-endpoint discard-dir {:db db}))))
+     (command/process-commands! conn mq-endpoint discard-dir opt-map))))
 
 (defn auto-deactivate-nodes!
   "Deactivate nodes which haven't had any activity (catalog/fact submission)
@@ -101,12 +104,12 @@
   {:pre [(map? db)
          (period? node-ttl)]}
   (try
-    (pl-utils/demarcate
+    (kitchensink/demarcate
       (format "sweep of stale nodes (threshold: %s)"
               (format-period node-ttl))
       (with-transacted-connection db
         (doseq [node (scf-store/stale-nodes (ago node-ttl))]
-          (send-command! "deactivate node" 1 (json/generate-string node)))))
+          (send-command! (command-names :deactivate-node) 1 (json/generate-string node)))))
     (catch Exception e
       (log/error e "Error while deactivating stale nodes"))))
 
@@ -116,7 +119,7 @@
   {:pre [(map? db)
          (period? node-purge-ttl)]}
   (try
-    (pl-utils/demarcate
+    (kitchensink/demarcate
       (format "purge deactivated nodes (threshold: %s)"
               (format-period node-purge-ttl))
       (with-transacted-connection db
@@ -130,7 +133,7 @@
   {:pre [(map? db)
          (period? report-ttl)]}
   (try
-    (pl-utils/demarcate
+    (kitchensink/demarcate
       (format "sweep of stale reports (threshold: %s)"
               (format-period report-ttl))
       (with-transacted-connection db
@@ -142,7 +145,7 @@
   "Compresses discarded message which are older than `dlo-compression-threshold`."
   [dlo dlo-compression-threshold]
   (try
-    (pl-utils/demarcate
+    (kitchensink/demarcate
       (format "compression of discarded messages (threshold: %s)"
               (format-period dlo-compression-threshold))
       (dlo/compress! dlo dlo-compression-threshold))
@@ -156,7 +159,7 @@
   [db]
   {:pre [(map? db)]}
   (try
-    (pl-utils/demarcate
+    (kitchensink/demarcate
       "database garbage collection"
       (with-transacted-connection db
         (scf-store/garbage-collect!)))
@@ -196,144 +199,20 @@
     (check-for-updates update-server db)
     (log/debug "Skipping update check on Puppet Enterprise")))
 
-(defn configure-commandproc-threads
-  "Update the supplied config map with the number of
-  command-processing threads to use. If no value exists in the config
-  map, default to half the number of CPUs. If only one CPU exists, we
-  will use one command-processing thread."
-  [config]
-  {:pre  [(map? config)]
-   :post [(map? %)
-          (pos? (get-in % [:command-processing :threads]))]}
-  (let [default-nthreads (-> (pl-utils/num-cpus)
-                             (/ 2)
-                             (int)
-                             (max 1))]
-    (update-in config [:command-processing :threads] #(or % default-nthreads))))
-
-(defn configure-web-server
-  "Update the supplied config map with information about the HTTP webserver to
-  start. This will specify client auth, and add a default host/port
-  http://puppetdb:8080 if none are supplied (and SSL is not specified)."
-  [config]
-  {:pre  [(map? config)]
-   :post [(map? %)]}
-  (assoc-in config [:jetty :client-auth] :need))
-
-(defn configure-gc-params
-  "Helper function that munges the supported permutations of our GC-related
-  `ttl` and interval settings (if present) from their config file
-  representation to our internal representation as Period objects."
-  [{:keys [database command-processing] :as config :or {database {}}}]
-  {:pre  [(map? config)]
-   :post [(map? %)
-          (= (dissoc database :gc-interval :report-ttl :node-purge-ttl :node-ttl :node-ttl-days)
-             (dissoc (:database %) :gc-interval :report-ttl :node-purge-ttl :node-ttl))
-          (period? (get-in % [:command-processing :dlo-compression-threshold]))
-          (every? period? (map (:database %) [:node-ttl :node-purge-ttl :report-ttl :gc-interval]))]}
-  (let [maybe-parse-period #(-?> % parse-period)
-        maybe-days #(-?> % days)
-        maybe-minutes #(-?> % minutes)
-        gc-interval-default (minutes 60)
-        dlo-compression-default (days 1)
-        ;; These defaults have to be actual periods rather than nil, because
-        ;; the user could explicitly specify 0, and we want to treat that the
-        ;; same
-        node-ttl-default (secs 0)
-        node-purge-ttl-default (secs 0)
-        report-ttl-default (days 14)
-        parsed-commandproc (update-in command-processing [:dlo-compression-threshold] #(or (maybe-parse-period %) dlo-compression-default))
-        parsed-database (-> database
-                            (update-in [:gc-interval] #(or (maybe-minutes %) gc-interval-default))
-                            (update-in [:report-ttl] #(or (maybe-parse-period %) report-ttl-default))
-                            (update-in [:node-purge-ttl] #(or (maybe-parse-period %) (secs 0)))
-                            (update-in [:node-ttl] #(or (maybe-parse-period %) (maybe-days (:node-ttl-days database)) node-ttl-default))
-                            (dissoc :node-ttl-days))]
-    (assoc config :database parsed-database :command-processing parsed-commandproc)))
-
-(defn configure-database
-  "Update the supplied config map with information about the database. Adds a
-  default hsqldb. If a single part of the database information is specified
-  (such as classname but not subprotocol), no defaults will be filled in."
-  [{:keys [database global] :as config}]
-  {:pre  [(map? config)]
-   :post [(map? config)]}
-  (let [vardir       (:vardir global)
-        default-db   {:classname   "org.hsqldb.jdbcDriver"
-                      :subprotocol "hsqldb"
-                      :subname     (format "file:%s;hsqldb.tx=mvcc;sql.syntax_pgs=true" (file vardir "db"))}]
-    (assoc config :database (or database default-db))))
-
-(defn normalize-product-name
-  "Checks that `product-name` is specified as a legal value, throwing an
-  exception if not. Returns `product-name` if it's okay."
-  [product-name]
-  {:pre [(string? product-name)]
-   :post [(= (string/lower-case product-name) %)]}
-  (let [lower-product-name (string/lower-case product-name)]
-    (when-not (#{"puppetdb" "pe-puppetdb"} lower-product-name)
-      (throw (IllegalArgumentException.
-               (format "product-name %s is illegal; either puppetdb or pe-puppetdb are allowed" product-name))))
-    lower-product-name))
-
-(defn validate-vardir
-  "Checks that `vardir` is specified, exists, and is writeable, throwing
-  appropriate exceptions if any condition is unmet."
-  [vardir]
-  (if-let [vardir (file vardir)]
-    (cond
-     (not (.isAbsolute vardir))
-     (throw (IllegalArgumentException.
-             (format "Vardir %s must be an absolute path." vardir)))
-
-     (not (.exists vardir))
-     (throw (java.io.FileNotFoundException.
-             (format "Vardir %s does not exist. Please create it and ensure it is writable." vardir)))
-
-     (not (.isDirectory vardir))
-     (throw (java.io.FileNotFoundException.
-             (format "Vardir %s is not a directory." vardir)))
-
-     (not (.canWrite vardir))
-     (throw (java.io.FileNotFoundException.
-             (format "Vardir %s is not writable." vardir))))
-
-    (throw (IllegalArgumentException.
-            "Required setting 'vardir' is not specified. Please set it to a writable directory.")))
-  vardir)
-
-(defn set-global-configuration!
-  "Store away global configuration"
-  [config]
-  {:pre  [(map? config)]
-   :post [(map? %)]}
-  (def configuration config)
-  config)
-
-(defn parse-config!
-  "Parses the given config file/directory and configures its various
-  subcomponents.
-
-  Also accepts an optional map argument 'initial-config'; if
-  provided, any initial values in this map will be included
-  in the resulting config map."
-  ([path]
-    (parse-config! path {}))
-  ([path initial-config]
-    {:pre [(string? path)
-           (map? initial-config)]}
-    (let [file (file path)]
-      (when-not (.canRead file)
-        (throw (IllegalArgumentException.
-                (format "Configuration path '%s' must exist and must be readable." path)))))
-
-    (-> (merge initial-config (inis-to-map path))
-        (configure-logging!)
-        (configure-commandproc-threads)
-        (configure-web-server)
-        (configure-database)
-        (configure-gc-params)
-        (set-global-configuration!))))
+(defn build-whitelist-authorizer
+  "Build a function that will authorize requests based on the supplied
+  certificate whitelist (see `cn-whitelist->authorizer` for more
+  details). Rejected requests are logged."
+  [whitelist]
+  {:pre  [(string? whitelist)]
+   :post [(fn? %)]}
+  (let [allowed? (kitchensink/cn-whitelist->authorizer whitelist)]
+    (fn [{:keys [ssl-client-cn] :as req}]
+      (if (allowed? req)
+        true
+        (do
+          (log/warnf "%s rejected by certificate whitelist %s" ssl-client-cn whitelist)
+          false)))))
 
 (defn on-shutdown
   "General cleanup when a shutdown request is received."
@@ -341,9 +220,8 @@
   ;; nothing much to do here for now, but let's at least log that we're shutting down.
   (log/info "Shutdown request received; puppetdb exiting."))
 
-
 (def supported-cli-options
-  [["-c" "--config" "Path to config.ini (required)"]
+  [["-c" "--config" "Path to config.ini or conf.d directory (required)"]
    ["-D" "--debug" "Enable debug mode" :default false :flag true]])
 
 (def required-cli-options
@@ -355,48 +233,48 @@
                                                       supported-cli-options
                                                       required-cli-options)
         initial-config                             {:debug (:debug options)}
-        {:keys [jetty database global command-processing]
-            :as config}                            (parse-config! (:config options) initial-config)
-        product-name                               (normalize-product-name (get global :product-name "puppetdb"))
-        vardir                                     (validate-vardir (:vardir global))
-        update-server                              (:update-server global "http://updates.puppetlabs.com/check-for-updates")
-        resource-query-limit                       (get global :resource-query-limit 20000)
+        {:keys [jetty database read-database global command-processing]
+            :as config}                            (conf/parse-config (:config options) initial-config)
+        product-name                               (:product-name global)
+        update-server                              (:update-server global)
         ;; TODO: revisit the choice of 20000 as a default value for event queries
-        event-query-limit                          (get global :event-query-limit 20000)
-        db                                         (pl-jdbc/pooled-datasource database)
+        event-query-limit                          (:event-query-limit global)
+        write-db                                   (pl-jdbc/pooled-datasource database)
+        read-db                                    (pl-jdbc/pooled-datasource (assoc read-database :read-only? true))
         gc-interval                                (get database :gc-interval)
         node-ttl                                   (get database :node-ttl)
         node-purge-ttl                             (get database :node-purge-ttl)
         report-ttl                                 (get database :report-ttl)
         dlo-compression-threshold                  (get command-processing :dlo-compression-threshold)
-        mq-dir                                     (str (file vardir "mq"))
+        mq-dir                                     (str (file (:vardir global) "mq"))
         discard-dir                                (file mq-dir "discarded")
-        globals                                    {:scf-db               db
+        globals                                    {:scf-read-db          read-db
+                                                    :scf-write-db         write-db
                                                     :command-mq           {:connection-string mq-addr
                                                                            :endpoint          mq-endpoint}
-                                                    :resource-query-limit resource-query-limit
                                                     :event-query-limit    event-query-limit
                                                     :update-server        update-server
                                                     :product-name         product-name}]
 
 
+    (utils/log-deprecated-jdk)
 
     (when (version)
       (log/info (format "PuppetDB version %s" (version))))
 
     ;; Add a shutdown hook where we can handle any required cleanup
-    (pl-utils/add-shutdown-hook! on-shutdown)
+    (kitchensink/add-shutdown-hook! on-shutdown)
 
     ;; Ensure the database is migrated to the latest version, and warn if it's
     ;; deprecated. We do this in a single connection because HSQLDB seems to
     ;; get confused if the database doesn't exist but we open and close a
     ;; connection without creating anything.
-    (sql/with-connection db
+    (sql/with-connection write-db
       (scf-store/warn-on-db-deprecation!)
       (migrate!))
 
     ;; Initialize database-dependent metrics
-    (pop/initialize-metrics db)
+    (pop/initialize-metrics write-db)
 
     (let [error         (promise)
           broker        (try
@@ -412,10 +290,11 @@
                           (log/info (format "Starting %d command processor threads" nthreads))
                           (vec (for [n (range nthreads)]
                                  (future (with-error-delivery error
-                                           (load-from-mq mq-addr mq-endpoint discard-dir db))))))
-          updater       (future (maybe-check-for-updates product-name update-server db))
+                                           (load-from-mq mq-addr mq-endpoint discard-dir {:db write-db
+                                                                                          :catalog-hash-debug-dir (:catalog-hash-debug-dir global)}))))))
+          updater       (future (maybe-check-for-updates product-name update-server read-db))
           web-app       (let [authorized? (if-let [wl (jetty :certificate-whitelist)]
-                                            (pl-utils/cn-whitelist->authorizer wl)
+                                            (build-whitelist-authorizer wl)
                                             (constantly true))
                               app         (server/build-app :globals globals :authorized? authorized?)]
                           (log/info "Starting query server")
@@ -434,12 +313,12 @@
                                   (when (pos? (to-secs report-ttl))
                                     (partial sweep-reports! report-ttl))]]
 
-        (gc-task #(apply perform-db-maintenance! db (remove nil? db-maintenance-tasks)))
+        (gc-task #(apply perform-db-maintenance! write-db (remove nil? db-maintenance-tasks)))
         (gc-task #(compress-dlo! dlo-compression-threshold discard-dir)))
 
       ;; Start debug REPL if necessary
       (let [{:keys [enabled type host port] :or {type "nrepl" host "localhost"}} (:repl config)]
-        (when (= "true" enabled)
+        (when (kitchensink/true-str? enabled)
           (log/warn (format "Starting %s server on port %d" type port))
           (start-repl type host port)))
 
